@@ -1,9 +1,9 @@
 box::use(
   shiny[testServer],
-  testthat[...],
+  testthat[expect_true, test_that],
 )
 box::use(
-  app/main[...],
+  app/main[server],
 )
 
 test_that("main server works", {
